@@ -1,6 +1,6 @@
 /*
  *
- * Copyright (C) 2019-2021 Intel Corporation
+ * Copyright (C) 2019-2025 Intel Corporation
  *
  * SPDX-License-Identifier: MIT
  *
@@ -14,9 +14,24 @@
 #include "zet_ddi.h"
 #include "zes_ddi.h"
 #include "ze_util.h"
+#include "ze_ddi_common.h"
+
+#ifndef ZEL_NULL_DRIVER_ID
+#define ZEL_NULL_DRIVER_ID 1
+#endif
 
 namespace driver
 {
+    extern ze_dditable_driver_t pCore;
+    extern zet_dditable_driver_t pTools;
+    extern zes_dditable_driver_t pSysman;
+    struct __zedlllocal BaseNullHandle : ze_handle_t {
+        BaseNullHandle() {
+            pCore = &driver::pCore;
+            pTools = &driver::pTools;
+            pSysman = &driver::pSysman;
+        }
+    };
     ///////////////////////////////////////////////////////////////////////////////
     class __zedlllocal context_t
     {
@@ -26,13 +41,24 @@ namespace driver
         ze_dditable_t   zeDdiTable = {};
         zet_dditable_t  zetDdiTable = {};
         zes_dditable_t  zesDdiTable = {};
+        std::vector<BaseNullHandle*> globalBaseNullHandle;
+        bool ddiExtensionRequested = false;
         context_t();
-        ~context_t() = default;
+        ~context_t() {
+            for (auto handle : globalBaseNullHandle) {
+                delete handle;
+            }
+        }
 
         void* get( void )
         {
             static uint64_t count = 0x80800000;
-            return reinterpret_cast<void*>( ++count );
+            if (ddiExtensionRequested) {
+                globalBaseNullHandle.push_back(new BaseNullHandle());
+                return reinterpret_cast<void*>(globalBaseNullHandle.back());
+            } else {
+                return reinterpret_cast<void*>( ++count );
+            }
         }
     };
 
